@@ -8,6 +8,7 @@ import getpass
 import json
 import logging
 import os
+import shlex
 import shutil
 import subprocess
 import sys
@@ -33,6 +34,7 @@ from wandb.apis import InternalApi, PublicApi
 from wandb.errors import ExecutionError, LaunchError
 from wandb.integration.magic import magic_install
 from wandb.sdk.launch.launch_add import _launch_add
+from wandb.sdk.launch.utils import construct_launch_spec
 from wandb.sdk.lib.wburls import wburls
 
 # from wandb.old.core import wandb_dir
@@ -663,6 +665,16 @@ def sync(
 @click.option("--program", default=None, help="Set sweep program")
 @click.option("--settings", default=None, help="Set sweep settings", hidden=True)
 @click.option("--update", default=None, help="Update pending sweep")
+@click.option(
+    "--queue",
+    "-q",
+    is_flag=False,
+    flag_value="default",
+    default=None,
+    help="Name of launch run queue to push sweep runs into. If supplied without "
+    "an argument (`--queue`), defaults to classic sweep behavior. Else, if "
+    "name supplied, specified run queue must exist under the project and entity supplied.",
+)
 @click.option(
     "--stop",
     is_flag=True,
@@ -699,6 +711,7 @@ def sweep(
     program,
     settings,
     update,
+    queue,
     stop,
     cancel,
     pause,
@@ -855,8 +868,57 @@ def sweep(
         or api.settings("project")
         or util.auto_project_name(config.get("program"))
     )
+
+    _launch_scheduler_spec = None
+    if queue is not None:
+        wandb.termlog("Using launch 🚀 queue: %s" % queue)
+
+        # Because the launch job spec below is the Scheduler, it
+        # will need to know the name of the sweep, which it wont
+        # know until it is created,so we use this placeholder
+        # and replace inside UpsertSweep in the backend (mutation.go)
+        _sweep_id_placeholder = "WANDB_SWEEP_ID"
+
+        # Launch job spec for the Scheduler
+        # TODO: Keep up to date with Launch Job Spec
+        _launch_scheduler_spec = json.dumps(
+            {
+                "queue": queue,
+                "run_spec": json.dumps(
+                    construct_launch_spec(
+                        os.getcwd(),  # uri,
+                        api,
+                        f"Scheduler.{_sweep_id_placeholder}",  # name,
+                        project,
+                        entity,
+                        None,  # docker_image,
+                        "local-process",  # resource,
+                        [
+                            "wandb",
+                            "scheduler",
+                            _sweep_id_placeholder,
+                            "--queue",
+                            queue,
+                            "--project",
+                            project,
+                        ],  # entry_point,
+                        None,  # version,
+                        None,  # params,
+                        None,  # resource_args,
+                        None,  # launch_config,
+                        None,  # cuda,
+                        None,  # run_id,
+                    )
+                ),
+            }
+        )
+
     sweep_id, warnings = api.upsert_sweep(
-        config, project=project, entity=entity, obj_id=sweep_obj_id
+        config,
+        project=project,
+        entity=entity,
+        obj_id=sweep_obj_id,
+        launch_scheduler=_launch_scheduler_spec,
     )
     util.handle_sweep_config_violations(warnings)
 
@@ -888,11 +950,18 @@ def sweep(
     if sweep_path.find(" ") >= 0:
         sweep_path = f'"{sweep_path}"'
 
-    wandb.termlog(
-        "Run sweep agent with: {}".format(
-            click.style("wandb agent %s" % sweep_path, fg="yellow")
+    if queue is not None:
+        wandb.termlog(
+            "If no launch agent is running, run launch agent with: {}".format(
+                click.style(f"wandb launch-agent -q {queue} -p {project}", fg="yellow")
+            )
+        )
+    else:
+        wandb.termlog(
+            "Run sweep agent with: {}".format(
+                click.style("wandb agent %s" % sweep_path, fg="yellow")
+            )
         )
-    )
     if controller:
         wandb.termlog("Starting wandb controller...")
         from wandb import controller as wandb_controller
@@ -958,7 +1027,7 @@ def sweep(
     "--resource",
     "-r",
     metavar="BACKEND",
-    default="local",
+    default=None,
     help="Execution resource to use for run. Supported values: 'local'."
     " If passed in, will override the resource value passed in using a config file."
     " Defaults to 'local'.",
@@ -1074,6 +1143,9 @@ def launch(
     else:
         resource_args = {}
 
+    if entry_point is not None:
+        entry_point = shlex.split(entry_point)
+
     if config is not None:
         config = util.load_json_yaml_dict(config)
         if config is None:
@@ -1081,6 +1153,11 @@ def launch(
     else:
         config = {}
 
+    if resource is None and config.get("resource") is not None:
+        resource = config.get("resource")
+    elif resource is None:
+        resource = "local-container"
+
     if (
         uri is None
         and docker_image is None
@@ -1135,22 +1212,38 @@ def launch(
 
 @cli.command(context_settings=CONTEXT, help="Run a W&B launch agent (Experimental)")
 @click.pass_context
-@click.argument("project", nargs=1, required=False)
+@click.option(
+    "--project",
+    "-p",
+    default=None,
+    help="Name of the project which the agent will watch. "
+    "If passed in, will override the project value passed in using a config file.",
+)
 @click.option(
     "--entity",
     "-e",
     default=None,
     help="The entity to use. Defaults to current logged-in user",
 )
-@click.option("--queues", "-q", default="default", help="The queue names to poll")
+@click.option("--queues", "-q", default=None, help="The queue names to poll")
 @click.option(
     "--max-jobs",
     "-j",
     default=None,
     help="The maximum number of launch jobs this agent can run in parallel. Defaults to 1. Set to -1 for no upper limit",
 )
+@click.option(
+    "--config", "-c", default=None, help="path to the agent config yaml to use"
+)
 @display_error
-def launch_agent(ctx, project=None, entity=None, queues=None, max_jobs=None):
+def launch_agent(
+    ctx,
+    project=None,
+    entity=None,
+    queues=None,
+    max_jobs=None,
+    config=None,
+):
     logger.info(
         f"=== Launch-agent called with kwargs {locals()}  CLI Version: {wandb.__version__} ==="
     )
@@ -1160,28 +1253,19 @@ def launch_agent(ctx, project=None, entity=None, queues=None, max_jobs=None):
     wandb.termlog(
         f"W&B launch is in an experimental state and usage APIs may change without warning. See {wburls.get('cli_launch')}"
     )
-    if project is None:
-        project = os.environ.get("WANDB_PROJECT")
-
-        if project is None:
-            raise LaunchError(
-                "You must specify a project name or set WANDB_PROJECT environment variable."
-            )
     api = _get_cling_api()
-    queues = queues.split(",")  # todo: check for none?
-    if api.api_key is None:
-        wandb.termlog("Login to W&B to use the launch agent feature")
-        ctx.invoke(login, no_offline=True)
-        api = _get_cling_api(reset=True)
-
-    if entity is None:
-        entity = api.default_entity
-    if max_jobs is None:
-        max_jobs = float(os.environ.get("WANDB_LAUNCH_MAX_JOBS", 1))
+    if queues is not None:
+        queues = queues.split(",")
+    agent_config, api = wandb_launch.resolve_agent_config(
+        api, entity, project, max_jobs, queues
+    )
+    if agent_config.get("project") is None:
+        raise LaunchError(
+            "You must specify a project name or set WANDB_PROJECT environment variable."
+        )
 
     wandb.termlog("Starting launch agent ✨")
-
-    wandb_launch.create_and_run_agent(api, entity, project, queues, max_jobs)
+    wandb_launch.create_and_run_agent(api, agent_config)
 
 
 @cli.command(context_settings=CONTEXT, help="Run the W&B agent")
@@ -1208,6 +1292,53 @@ def agent(ctx, project, entity, count, sweep_id):
     #                'args': ['--max_epochs=10']})
 
 
+@cli.command(
+    context_settings=CONTEXT, help="Run a W&B launch sweep scheduler (Experimental)"
+)
+@click.pass_context
+@click.option(
+    "--project",
+    "-p",
+    default=None,
+    help="Name of the project which the agent will watch. "
+    "If passed in, will override the project value passed in using a config file.",
+)
+@click.option(
+    "--entity",
+    "-e",
+    default=None,
+    help="The entity to use. Defaults to current logged-in user",
+)
+@click.option(
+    "--queue",
+    "-q",
+    default=None,
+    help="The queue to push sweep jobs to.",
+)
+@click.argument("sweep_id")
+@display_error
+def scheduler(
+    ctx,
+    project,
+    entity,
+    queue,
+    sweep_id,
+):
+    api = _get_cling_api()
+    if api.api_key is None:
+        wandb.termlog("Login to W&B to use the sweep scheduler feature")
+        ctx.invoke(login, no_offline=True)
+        api = _get_cling_api(reset=True)
+
+    wandb.termlog("Starting a Launch Scheduler 🚀")
+    from wandb.sdk.launch.sweeps import load_scheduler
+
+    _scheduler = load_scheduler("sweep")(
+        api, entity=entity, project=project, queue=queue, sweep_id=sweep_id
+    )
+    _scheduler.start()
+
+
 @cli.command(context_settings=CONTEXT, help="Run the W&B local sweep controller")
 @click.option("--verbose", is_flag=True, default=False, help="Display verbose output")
 @click.argument("sweep_id")
@@ -1401,7 +1532,9 @@ def docker(
 
 
 @cli.command(
-    context_settings=RUN_CONTEXT, help="Launch local W&B container (Experimental)"
+    context_settings=RUN_CONTEXT,
+    help="Start a local W&B container (deprecated, see wandb server --help)",
+    hidden=True,
 )
 @click.pass_context
 @click.option("--port", "-p", default="8080", help="The host port to bind W&B local on")
@@ -1415,21 +1548,55 @@ def docker(
     "--upgrade", is_flag=True, default=False, help="Upgrade to the most recent version"
 )
 @click.option(
-    "--edge", is_flag=True, default=False, help="Run the bleading edge", hidden=True
+    "--edge", is_flag=True, default=False, help="Run the bleeding edge", hidden=True
 )
 @display_error
-def local(ctx, port, env, daemon, upgrade, edge):
+def local(ctx, *args, **kwargs):
+    wandb.termwarn("`wandb local` has been replaced with `wandb server start`.")
+    ctx.invoke(start, *args, **kwargs)
+
+
+@cli.group(help="Commands for operating a local W&B server")
+def server():
+    pass
+
+
+@server.command(context_settings=RUN_CONTEXT, help="Start a local W&B server")
+@click.pass_context
+@click.option(
+    "--port", "-p", default="8080", help="The host port to bind W&B server on"
+)
+@click.option(
+    "--env", "-e", default=[], multiple=True, help="Env vars to pass to wandb/local"
+)
+@click.option(
+    "--daemon/--no-daemon", default=True, help="Run or don't run in daemon mode"
+)
+@click.option(
+    "--upgrade",
+    is_flag=True,
+    default=False,
+    help="Upgrade to the most recent version",
+    hidden=True,
+)
+@click.option(
+    "--edge", is_flag=True, default=False, help="Run the bleeding edge", hidden=True
+)
+@display_error
+def start(ctx, port, env, daemon, upgrade, edge):
     api = InternalApi()
     if not find_executable("docker"):
         raise ClickException("Docker not installed, install it from https://docker.com")
-    if wandb.docker.image_id("wandb/local") != wandb.docker.image_id_from_registry(
+    local_image_sha = wandb.docker.image_id("wandb/local").split("wandb/local")[-1]
+    registry_image_sha = wandb.docker.image_id_from_registry("wandb/local").split(
         "wandb/local"
-    ):
+    )[-1]
+    if local_image_sha != registry_image_sha:
         if upgrade:
             subprocess.call(["docker", "pull", "wandb/local"])
         else:
             wandb.termlog(
-                "A new version of W&B local is available, upgrade by calling `wandb local --upgrade`"
+                "A new version of the W&B server is available, upgrade by calling `wandb server start --upgrade`"
             )
     running = subprocess.check_output(
         ["docker", "ps", "--filter", "name=wandb-local", "--format", "{{.ID}}"]
@@ -1459,7 +1626,7 @@ def local(ctx, port, env, daemon, upgrade, edge):
         "--name",
         "wandb-local",
     ] + env_vars
-    host = "http://localhost:%s" % port
+    host = f"http://localhost:{port}"
     api.set_setting("base_url", host, globally=True, persist=True)
     if daemon:
         command += ["-d"]
@@ -1488,6 +1655,13 @@ def local(ctx, port, env, daemon, upgrade, edge):
                 ctx.invoke(login, host=host)
 
 
+@server.command(context_settings=RUN_CONTEXT, help="Stop a local W&B server")
+def stop():
+    if not find_executable("docker"):
+        raise ClickException("Docker not installed, install it from https://docker.com")
+    subprocess.call(["docker", "stop", "wandb-local"])
+
+
 @cli.group(help="Commands for interacting with artifacts")
 def artifact():
     pass
