@@ -112,7 +112,12 @@ def _json_helper(val, artifact):
         return res
 
     if hasattr(val, "tolist"):
-        return _json_helper(val.tolist(), artifact)
+        py_val = val.tolist()
+        if val.__class__.__name__ == "datetime64" and isinstance(py_val, int):
+            # when numpy datetime64 .tolist() returns an int, it is nanoseconds.
+            # need to convert to milliseconds
+            return _json_helper(py_val / int(1e6), artifact)
+        return _json_helper(py_val, artifact)
     elif hasattr(val, "item"):
         return _json_helper(val.item(), artifact)
 
@@ -439,6 +444,7 @@ class Table(Media):
         return self._eq_debug(other)
 
     def add_row(self, *row):
+        """add_row is deprecated. Please use add_data"""
         logging.warning("add_row is deprecated, use add_data")
         self.add_data(*row)
 
@@ -494,7 +500,18 @@ class Table(Media):
         # separate this method for easier testing
         if max_rows is None:
             max_rows = Table.MAX_ROWS
-        if len(self.data) > max_rows and warn:
+        n_rows = len(self.data)
+        if n_rows > max_rows and warn:
+            if wandb.run and (
+                wandb.run.settings.table_raise_on_max_row_limit_exceeded
+                or wandb.run.settings.strict
+            ):
+                raise ValueError(
+                    f"Table row limit exceeded: table has {n_rows} rows, limit is {max_rows}. "
+                    f"To increase the maximum number of allowed rows in a wandb.Table, override "
+                    f"the limit with `wandb.Table.MAX_ARTIFACT_ROWS = X` and try again. Note: "
+                    f"this may cause slower queries in the W&B UI."
+                )
             logging.warning("Truncating wandb.Table object to %i rows." % max_rows)
         return {"columns": self.columns, "data": self.data[:max_rows]}
 
@@ -520,6 +537,7 @@ class Table(Media):
         data = []
         column_types = None
         np_deserialized_columns = {}
+        timestamp_column_indices = set()
         if json_obj.get("column_types") is not None:
             column_types = _dtypes.TypeRegistry.type_from_dict(
                 json_obj["column_types"], source_artifact
@@ -533,6 +551,14 @@ class Table(Media):
                     for t in col_type.params["allowed_types"]:
                         if isinstance(t, _dtypes.NDArrayType):
                             ndarray_type = t
+                        elif isinstance(t, _dtypes.TimestampType):
+                            timestamp_column_indices.add(
+                                json_obj["columns"].index(col_name)
+                            )
+
+                elif isinstance(col_type, _dtypes.TimestampType):
+                    timestamp_column_indices.add(json_obj["columns"].index(col_name))
+
                 if (
                     ndarray_type is not None
                     and ndarray_type._get_serialization_path() is not None
@@ -554,7 +580,11 @@ class Table(Media):
             row_data = []
             for c_ndx, item in enumerate(row):
                 cell = item
-                if c_ndx in np_deserialized_columns:
+                if c_ndx in timestamp_column_indices and isinstance(item, (int, float)):
+                    cell = datetime.datetime.fromtimestamp(
+                        item / 1000, tz=datetime.timezone.utc
+                    )
+                elif c_ndx in np_deserialized_columns:
                     cell = np_deserialized_columns[c_ndx][r_ndx]
                 elif isinstance(item, dict) and "_type" in item:
                     obj = WBValue.init_from_json(item, source_artifact)
@@ -1745,7 +1775,12 @@ class _ImageFileType(_dtypes.Type):
     types = [Image]
 
     def __init__(
-        self, box_layers=None, box_score_keys=None, mask_layers=None, class_map=None
+        self,
+        box_layers=None,
+        box_score_keys=None,
+        mask_layers=None,
+        class_map=None,
+        **kwargs,
     ):
         box_layers = box_layers or {}
         box_score_keys = box_score_keys or []
