@@ -1,8 +1,10 @@
+import json
 import logging
 import os
 import subprocess
 from typing import Any, Dict, List, Optional, Tuple, Union
 
+from dockerpycreds.utils import find_executable  # type: ignore
 import requests
 from wandb.docker import auth
 from wandb.docker import www_authenticate
@@ -39,8 +41,11 @@ def is_buildx_installed() -> bool:
     global _buildx_installed
     if _buildx_installed is not None:
         return _buildx_installed
-    help_output = shell(["buildx", "--help"])
-    _buildx_installed = help_output is not None and "buildx" in help_output
+    if not find_executable("docker"):
+        _buildx_installed = False
+    else:
+        help_output = shell(["buildx", "--help"])
+        _buildx_installed = help_output is not None and "buildx" in help_output
     return _buildx_installed
 
 
@@ -199,9 +204,13 @@ def image_id(image_name: str) -> Optional[str]:
     if "@sha256:" in image_name:
         return image_name
     else:
-        return shell(
-            ["inspect", image_name, "--format", "{{index .RepoDigests 0}}"]
-        ) or image_id_from_registry(image_name)
+        digests = shell(["inspect", image_name, "--format", "{{json .RepoDigests}}"])
+        try:
+            if digests is None:
+                raise ValueError()
+            return json.loads(digests)[0]
+        except (ValueError, IndexError):
+            return image_id_from_registry(image_name)
 
 
 def get_image_uid(image_name: str) -> int:
