@@ -4,37 +4,43 @@ import os
 import queue
 import shutil
 import threading
-from typing import Any, Callable, NamedTuple, Union
+from typing import cast, NamedTuple, Optional, TYPE_CHECKING, Union
 
-from wandb.filesync import step_upload
+from wandb.filesync import dir_watcher, step_upload
 import wandb.util
 
+if TYPE_CHECKING:
+    import tempfile
+    from wandb.filesync import stats
+    from wandb.sdk.interface import artifacts
+    from wandb.sdk.internal import artifacts as internal_artifacts, internal_api
+
 
 class RequestUpload(NamedTuple):
     path: str
-    save_name: str
-    artifact_id: str
+    save_name: dir_watcher.SaveName
+    artifact_id: Optional[str]
     copy: bool
     use_prepare_flow: bool
-    save_fn: Callable[..., Any]
-    digest: Any
+    save_fn: Optional[step_upload.SaveFn]
+    digest: Optional[str]
 
 
 class RequestStoreManifestFiles(NamedTuple):
-    manifest: Any
+    manifest: "artifacts.ArtifactManifest"
     artifact_id: str
-    save_fn: Callable[..., Any]
+    save_fn: "internal_artifacts.SaveFn"
 
 
 class RequestCommitArtifact(NamedTuple):
     artifact_id: str
     finalize: bool
-    before_commit: Callable[..., Any]
-    on_commit: Callable[..., Any]
+    before_commit: Optional[step_upload.PreCommitFn]
+    on_commit: Optional[step_upload.PostCommitFn]
 
 
 class RequestFinish(NamedTuple):
-    callback: Callable[..., Any]
+    callback: Optional[step_upload.OnRequestFinishFn]
 
 
 Event = Union[
@@ -45,12 +51,12 @@ Event = Union[
 class StepChecksum:
     def __init__(
         self,
-        api,
-        tempdir,
+        api: "internal_api.Api",
+        tempdir: "tempfile.TemporaryDirectory",
         request_queue: "queue.Queue[Event]",
         output_queue: "queue.Queue[step_upload.Event]",
-        stats,
-    ):
+        stats: "stats.Stats",
+    ) -> None:
         self._api = api
         self._tempdir = tempdir
         self._request_queue = request_queue
@@ -60,7 +66,7 @@ class StepChecksum:
         self._thread = threading.Thread(target=self._thread_body)
         self._thread.daemon = True
 
-    def _thread_body(self):
+    def _thread_body(self) -> None:
         while True:
             req = self._request_queue.get()
             if isinstance(req, RequestUpload):
@@ -76,7 +82,7 @@ class StepChecksum:
                         # large files: https://bugs.python.org/issue43743
                         shutil.copy2(req.path, path)
                     except OSError:
-                        shutil._USE_CP_SENDFILE = False
+                        shutil._USE_CP_SENDFILE = False  # type: ignore[attr-defined]
                         shutil.copy2(req.path, path)
                 checksum = None
                 if req.use_prepare_flow:
@@ -101,18 +107,25 @@ class StepChecksum:
                 for entry in req.manifest.entries.values():
                     if entry.local_path:
                         # This stupid thing is needed so the closure works correctly.
-                        def make_save_fn_with_entry(save_fn, entry):
+                        def make_save_fn_with_entry(
+                            save_fn: "internal_artifacts.SaveFn",
+                            entry: "artifacts.ArtifactEntry",
+                        ) -> step_upload.SaveFn:
                             return lambda progress_callback: save_fn(
                                 entry, progress_callback
                             )
 
                         self._stats.init_file(
-                            entry.local_path, entry.size, is_artifact_file=True
+                            entry.local_path,
+                            cast(int, entry.size),
+                            is_artifact_file=True,
                         )
                         self._output_queue.put(
                             step_upload.RequestUpload(
                                 entry.local_path,
-                                entry.path,
+                                dir_watcher.SaveName(
+                                    entry.path
+                                ),  # typecast might not be legit
                                 req.artifact_id,
                                 entry.digest,
                                 False,
@@ -133,11 +146,11 @@ class StepChecksum:
 
         self._output_queue.put(step_upload.RequestFinish(req.callback))
 
-    def start(self):
+    def start(self) -> None:
         self._thread.start()
 
-    def is_alive(self):
+    def is_alive(self) -> bool:
         return self._thread.is_alive()
 
-    def finish(self):
+    def finish(self) -> None:
         self._request_queue.put(RequestFinish(None))
