@@ -1,10 +1,16 @@
 import collections
-import os
 import logging
+import os
 import threading
+from typing import Optional, TYPE_CHECKING
 
 import wandb
 
+if TYPE_CHECKING:
+    import queue
+    from wandb.filesync import dir_watcher, stats, step_upload
+    from wandb.sdk.internal import file_stream, internal_api
+
 EventJobDone = collections.namedtuple("EventJobDone", ("job", "success"))
 logger = logging.getLogger(__name__)
 
@@ -12,19 +18,19 @@ logger = logging.getLogger(__name__)
 class UploadJob(threading.Thread):
     def __init__(
         self,
-        done_queue,
-        stats,
-        api,
-        file_stream,
-        silent,
-        save_name,
-        path,
-        artifact_id,
-        md5,
-        copied,
-        save_fn,
-        digest,
-    ):
+        done_queue: "queue.Queue[step_upload.Event]",
+        stats: "stats.Stats",
+        api: "internal_api.Api",
+        file_stream: "file_stream.FileStreamApi",
+        silent: bool,
+        save_name: "dir_watcher.SaveName",
+        path: "dir_watcher.PathStr",
+        artifact_id: Optional[str],
+        md5: Optional[str],
+        copied: bool,
+        save_fn: Optional["step_upload.SaveFn"],
+        digest: Optional[str],
+    ) -> None:
         """A file upload thread.
 
         Arguments:
@@ -50,7 +56,7 @@ class UploadJob(threading.Thread):
         self.digest = digest
         super().__init__()
 
-    def run(self):
+    def run(self) -> None:
         success = False
         try:
             success = self.push()
@@ -61,12 +67,7 @@ class UploadJob(threading.Thread):
             if success:
                 self._file_stream.push_success(self.artifact_id, self.save_name)
 
-    def push(self):
-        try:
-            size = os.path.getsize(self.save_path)
-        except OSError:
-            size = 0
-
+    def push(self) -> bool:
         if self.save_fn:
             # Retry logic must happen in save_fn currently
             try:
@@ -80,7 +81,7 @@ class UploadJob(threading.Thread):
                 message = str(e)
                 # TODO: this is usually XML, but could be JSON
                 if hasattr(e, "response"):
-                    message = e.response.content
+                    message = e.response.content  # type: ignore[attr-defined]
                 wandb.termerror(
                     'Error uploading "{}": {}, {}'.format(
                         self.save_path, type(e).__name__, message
@@ -148,5 +149,5 @@ class UploadJob(threading.Thread):
                 return False
         return True
 
-    def progress(self, total_bytes):
+    def progress(self, total_bytes: int) -> None:
         self._stats.update_uploaded_file(self.save_name, total_bytes)
