@@ -2,6 +2,7 @@ from wandb.plot.bar import bar
 from wandb.plot.confusion_matrix import confusion_matrix
 from wandb.plot.histogram import histogram
 from wandb.plot.line import line
+from wandb.plot.line_series import line_series
 from wandb.plot.pr_curve import pr_curve
 from wandb.plot.roc_curve import roc_curve
 from wandb.plot.scatter import scatter
@@ -15,4 +16,5 @@ __all__ = [
     "roc_curve",
     "pr_curve",
     "confusion_matrix",
+    "line_series",
 ]
