@@ -1,5 +1,3 @@
-#
-# -*- coding: utf-8 -*-
 """
 module sdk
 """
@@ -8,14 +6,13 @@ from . import wandb_helper as helper  # noqa: F401
 from .wandb_alerts import AlertLevel  # noqa: F401
 from .wandb_artifacts import Artifact  # noqa: F401
 from .wandb_config import Config  # noqa: F401
-from .wandb_history import History  # noqa: F401
-from .wandb_init import init  # noqa: F401
+from .wandb_init import _attach, init  # noqa: F401
 from .wandb_login import login  # noqa: F401
 from .wandb_require import require  # noqa: F401
 from .wandb_run import finish  # noqa: F401
 from .wandb_save import save  # noqa: F401
 from .wandb_settings import Settings  # noqa: F401
-from .wandb_setup import setup  # noqa: F401
+from .wandb_setup import setup, teardown  # noqa: F401
 from .wandb_summary import Summary  # noqa: F401
 from .wandb_sweep import controller, sweep  # noqa: F401
 from .wandb_watch import unwatch, watch  # noqa: F401
