@@ -15,8 +15,8 @@ if TYPE_CHECKING:
 CAN_INFER_IMAGE_AND_VIDEO = sys.version_info.major == 3 and sys.version_info.minor >= 5
 
 
-class ValidationDataLogger(object):
-    """ValidationDataLogger helps to develop integrations which log  model predictions.
+class ValidationDataLogger:
+    """Logs validation data as a wandb.Table.
 
     ValidationDataLogger is intended to be used inside of library integrations
     in order to facilitate the process of optionally building a validation dataset
@@ -48,45 +48,41 @@ class ValidationDataLogger(object):
         """Initializes a new ValidationDataLogger.
 
         Args:
-            inputs (Sequence | Dict[str, Sequence]): a list of input vectors or
-                dictionary of lists of input vectors (used if the model has
-                multiple named inputs)
-            targets (Sequence | Dict[str, Sequence], optional): a list of target
-                vectors or dictionary of lists of target vectors (used if the
-                model has multiple named targets/putputs). Defaults to None.
-                `targets` and `indexes` cannot both be None
-            indexes (List[wandb.data_types._TableIndex], optional): An ordered
-                list of wandb.data_types._TableIndex mapping the input items to
-                their source table. This is most commonly retrieved by using
-                indexes = my_data_table.get_index().Defaults to None. `targets`
-                and `indexes` cannot both be None.
-            validation_row_processor (Callable, optional): a function to apply to
-                the validation data, commonly used to visualize the data. The
-                function will receive an ndx (int) and a row (dict). If `inputs`
-                is a list, then row["input"] will be the input data for the row.
-                Else, it will be keyed based on the name of the input slot
+            inputs: A list of input vectors or dictionary of lists of input vectors
+                (used if the model has multiple named inputs)
+            targets: A list of target vectors or dictionary of lists of target vectors
+                (used if the model has multiple named targets/putputs). Defaults to `None`.
+                `targets` and `indexes` cannot both be `None`.
+            indexes: An ordered list of `wandb.data_types._TableIndex` mapping the
+                input items to their source table. This is most commonly retrieved by using
+                `indexes = my_data_table.get_index()`. Defaults to `None`. `targets`
+                and `indexes` cannot both be `None`.
+            validation_row_processor: A function to apply to the validation data,
+                commonly used to visualize the data. The function will receive an `ndx` (`int`)
+                and a `row` (`dict`). If `inputs` is a list, then `row["input"]` will be the input
+                data for the row. Else, it will be keyed based on the name of the input slot
                 (corresponding to `inputs`). If `targets` is a list, then
-                row["target"] will be the target data for the row. Else, it will
+                `row["target"]` will be the target data for the row. Else, it will
                 be keyed based on `targets`. For example, if your input data is a
-                single ndarray, but you wish to visualize the data as an Image,
+                single ndarray, but you wish to visualize the data as an image,
                 then you can provide `lambda ndx, row: {"img": wandb.Image(row["input"])}`
-                as the processor. If None, we will try to guess the appropriate processor.
-                Ignored if log_evaluation is False or val_keys are present. Defaults to None.
-            prediction_row_processor (Callable, optional): same as validation_row_processor,
-                but applied to the model's output. `row["output"]` will contain the
-                results of the model output. Defaults to None.
-            input_col_name (str, optional): the name to use for the input column.
-                Defaults to "input".
-            target_col_name (str, optional): the name to use for the target column.
-                Defaults to "target".
-            table_name (str, optional): the name to use for the validation table.
-                Defaults to "wb_validation_data".
-            artifact_type (str, optional): the artifact type to use for the validation data.
-                Defaults to "validation_dataset".
-            class_labels (List[str], optional): Optional list of lables to use in the inferfed
-                processesors. If the model's `target` or `output` is inferred to be a class,
-                we will attempt to map the class to these labels. Defaults to None.
-            infer_missing_processors (bool, optional): Determines if processors are inferred if
+                as the processor. If `None`, we will try to guess the appropriate processor.
+                Ignored if `log_evaluation` is `False` or `val_keys` are present. Defaults to `None`.
+            prediction_row_processor: Same as validation_row_processor, but applied to the
+                model's output. `row["output"]` will contain the results of the model output.
+                Defaults to `None`.
+            input_col_name: The name to use for the input column.
+                Defaults to `"input"`.
+            target_col_name: The name to use for the target column.
+                Defaults to `"target"`.
+            table_name: The name to use for the validation table.
+                Defaults to `"wb_validation_data"`.
+            artifact_type: The artifact type to use for the validation data.
+                Defaults to `"validation_dataset"`.
+            class_labels: Optional list of lables to use in the inferred
+                processors. If the model's `target` or `output` is inferred to be a class,
+                we will attempt to map the class to these labels. Defaults to `None`.
+            infer_missing_processors: Determines if processors are inferred if
                 they are missing. Defaults to True.
         """
         class_labels_table: Optional["wandb.Table"]
@@ -266,7 +262,8 @@ def _infer_single_example_keyed_processor(
         and shape[0] == len(class_labels_table.data)
     ):
         np = wandb.util.get_module(
-            "numpy", required="Infering processors require numpy",
+            "numpy",
+            required="Infering processors require numpy",
         )
         # Assume these are logits
         class_names = class_labels_table.get_column("label")
@@ -280,7 +277,7 @@ def _infer_single_example_keyed_processor(
         # )
 
         values = np.unique(example)
-        is_one_hot = len(values) == 2 and set(values) == set([0, 1])
+        is_one_hot = len(values) == 2 and set(values) == {0, 1}
         if not is_one_hot:
             processors["score"] = lambda n, d, p: {
                 class_names[i]: d[i] for i in range(shape[0])
@@ -300,7 +297,8 @@ def _infer_single_example_keyed_processor(
             processors["val"] = lambda n, d, p: d[0]
     elif len(shape) == 1:
         np = wandb.util.get_module(
-            "numpy", required="Infering processors require numpy",
+            "numpy",
+            required="Infering processors require numpy",
         )
         # This could be anything
         if shape[0] <= 10:
@@ -314,7 +312,7 @@ def _infer_single_example_keyed_processor(
         processors["argmax"] = lambda n, d, p: np.argmax(d)
 
         values = np.unique(example)
-        is_one_hot = len(values) == 2 and set(values) == set([0, 1])
+        is_one_hot = len(values) == 2 and set(values) == {0, 1}
         if not is_one_hot:
             processors["argmin"] = lambda n, d, p: np.argmin(d)
     elif len(shape) == 2 and CAN_INFER_IMAGE_AND_VIDEO:
@@ -359,9 +357,11 @@ def _infer_validation_row_processor(
         for key in example_input:
             key_processors = _infer_single_example_keyed_processor(example_input[key])
             for p_key in key_processors:
-                single_processors["{}:{}".format(key, p_key)] = _bind(
+                single_processors[f"{key}:{p_key}"] = _bind(
                     lambda ndx, row, key_processor, key: key_processor(
-                        ndx, row[key], None,
+                        ndx,
+                        row[key],
+                        None,
                     ),
                     key_processor=key_processors[p_key],
                     key=key,
@@ -370,9 +370,11 @@ def _infer_validation_row_processor(
         key = input_col_name
         key_processors = _infer_single_example_keyed_processor(example_input)
         for p_key in key_processors:
-            single_processors["{}:{}".format(key, p_key)] = _bind(
+            single_processors[f"{key}:{p_key}"] = _bind(
                 lambda ndx, row, key_processor, key: key_processor(
-                    ndx, row[key], None,
+                    ndx,
+                    row[key],
+                    None,
                 ),
                 key_processor=key_processors[p_key],
                 key=key,
@@ -384,9 +386,11 @@ def _infer_validation_row_processor(
                 example_target[key], class_labels_table
             )
             for p_key in key_processors:
-                single_processors["{}:{}".format(key, p_key)] = _bind(
+                single_processors[f"{key}:{p_key}"] = _bind(
                     lambda ndx, row, key_processor, key: key_processor(
-                        ndx, row[key], None,
+                        ndx,
+                        row[key],
+                        None,
                     ),
                     key_processor=key_processors[p_key],
                     key=key,
@@ -399,7 +403,7 @@ def _infer_validation_row_processor(
             example_input if not isinstance(example_input, dict) else None,
         )
         for p_key in key_processors:
-            single_processors["{}:{}".format(key, p_key)] = _bind(
+            single_processors[f"{key}:{p_key}"] = _bind(
                 lambda ndx, row, key_processor, key: key_processor(
                     ndx,
                     row[key],
@@ -433,9 +437,11 @@ def _infer_prediction_row_processor(
                 example_prediction[key], class_labels_table
             )
             for p_key in key_processors:
-                single_processors["{}:{}".format(key, p_key)] = _bind(
+                single_processors[f"{key}:{p_key}"] = _bind(
                     lambda ndx, row, key_processor, key: key_processor(
-                        ndx, row[key], None,
+                        ndx,
+                        row[key],
+                        None,
                     ),
                     key_processor=key_processors[p_key],
                     key=key,
@@ -448,7 +454,7 @@ def _infer_prediction_row_processor(
             example_input if not isinstance(example_input, dict) else None,
         )
         for p_key in key_processors:
-            single_processors["{}:{}".format(key, p_key)] = _bind(
+            single_processors[f"{key}:{p_key}"] = _bind(
                 lambda ndx, row, key_processor, key: key_processor(
                     ndx,
                     row[key],
