@@ -9,7 +9,7 @@ import logging
 import os
 from shlex import quote
 import tempfile
-from typing import Any, Dict, Optional, Tuple
+from typing import Any, Dict, List, Optional
 
 import wandb
 from wandb.apis.internal import Api
@@ -53,6 +53,7 @@ class LaunchProject:
         resource: str,
         resource_args: Dict[str, Any],
         cuda: Optional[bool],
+        run_id: Optional[str],
     ):
         if uri is not None and utils.is_bare_wandb_uri(uri):
             uri = api.settings("base_url") + uri
@@ -61,8 +62,8 @@ class LaunchProject:
         self.api = api
         self.launch_spec = launch_spec
         self.target_entity = target_entity
-        self.target_project = target_project
-        self.name = name
+        self.target_project = target_project.lower()
+        self.name = name  # TODO: replace with run_id
         self.build_image: bool = docker_config.get("build_image", False)
         self.python_version: Optional[str] = docker_config.get("python_version")
         self.cuda_version: Optional[str] = docker_config.get("cuda_version")
@@ -78,22 +79,21 @@ class LaunchProject:
         self.override_args: Dict[str, Any] = overrides.get("args", {})
         self.override_config: Dict[str, Any] = overrides.get("run_config", {})
         self.override_artifacts: Dict[str, Any] = overrides.get("artifacts", {})
+        self.override_entrypoint: Optional[EntryPoint] = None
         self.resource = resource
         self.resource_args = resource_args
         self.deps_type: Optional[str] = None
         self.cuda = cuda
         self._runtime: Optional[str] = None
-        self.run_id = generate_id()
+        self.run_id = run_id or generate_id()
         self._entry_points: Dict[
             str, EntryPoint
         ] = {}  # todo: keep multiple entrypoint support?
-        if (
-            "entry_point" in overrides
-            and overrides["entry_point"] is not None
-            and overrides["entry_point"] != ""
-        ):
+        if overrides.get("entry_point") is not None:
             _logger.info("Adding override entry point")
-            self.add_entry_point(overrides["entry_point"])
+            self.override_entrypoint = self.add_entry_point(
+                overrides.get("entry_point")  # type: ignore
+            )
         if self.uri is None:
             if self.docker_image is None:
                 raise LaunchError("Run requires a URI or a docker image")
@@ -165,28 +165,12 @@ class LaunchProject:
             return None
         return list(self._entry_points.values())[0]
 
-    def add_entry_point(self, entry_point: str) -> "EntryPoint":
+    def add_entry_point(self, command: List[str]) -> "EntryPoint":
         """Adds an entry point to the project."""
-        _, file_extension = os.path.splitext(entry_point)
-        ext_to_cmd = {".py": "python", ".sh": os.environ.get("SHELL", "bash")}
-        if file_extension in ext_to_cmd:
-            command = f"{ext_to_cmd[file_extension]} {quote(entry_point)}"
-            new_entrypoint = EntryPoint(name=entry_point, command=command)
-            self._entry_points[entry_point] = new_entrypoint
-            return new_entrypoint
-        raise ExecutionError(
-            "Could not find {0} among entry points {1} or interpret {0} as a "
-            "runnable script. Supported script file extensions: "
-            "{2}".format(
-                entry_point, list(self._entry_points.keys()), list(ext_to_cmd.keys())
-            )
-        )
-
-    def get_entry_point(self, entry_point: str) -> "EntryPoint":
-        """Gets the entrypoint if its set, or adds it and returns the entrypoint."""
-        if entry_point in self._entry_points:
-            return self._entry_points[entry_point]
-        return self.add_entry_point(entry_point)
+        entry_point = command[-1]
+        new_entrypoint = EntryPoint(name=entry_point, command=command)
+        self._entry_points[entry_point] = new_entrypoint
+        return new_entrypoint
 
     def _fetch_project_local(self, internal_api: Api) -> None:
         """Fetch a project (either wandb run or git repo) into a local directory, returning the path to the local project directory."""
@@ -202,7 +186,7 @@ class LaunchProject:
             run_info = utils.fetch_wandb_project_run_info(
                 source_entity, source_project, source_run_name, internal_api
             )
-            entry_point = run_info.get("codePath") or run_info["program"]
+            program_name = run_info.get("codePath") or run_info["program"]
 
             if run_info.get("cudaVersion"):
                 original_cuda_version = ".".join(run_info["cudaVersion"].split(".")[:2])
@@ -254,18 +238,18 @@ class LaunchProject:
                 if patch:
                     utils.apply_patch(patch, self.project_dir)
                 # For cases where the entry point wasn't checked into git
-                if not os.path.exists(os.path.join(self.project_dir, entry_point)):
+                if not os.path.exists(os.path.join(self.project_dir, program_name)):
                     downloaded_entrypoint = utils.download_entry_point(
                         source_entity,
                         source_project,
                         source_run_name,
                         internal_api,
-                        entry_point,
+                        program_name,
                         self.project_dir,
                     )
                     if not downloaded_entrypoint:
                         raise LaunchError(
-                            f"Entrypoint: {entry_point} does not exist, "
+                            f"Entrypoint file: {program_name} does not exist, "
                             "and could not be downloaded. Please specify the entrypoint for this run."
                         )
                     # if the entrypoint is downloaded and inserted into the project dir
@@ -274,10 +258,10 @@ class LaunchProject:
 
             if (
                 "_session_history.ipynb" in os.listdir(self.project_dir)
-                or ".ipynb" in entry_point
+                or ".ipynb" in program_name
             ):
-                entry_point = utils.convert_jupyter_notebook_to_script(
-                    entry_point, self.project_dir
+                program_name = utils.convert_jupyter_notebook_to_script(
+                    program_name, self.project_dir
                 )
 
             # Download any frozen requirements
@@ -291,8 +275,15 @@ class LaunchProject:
 
             # Specify the python runtime for jupyter2docker
             self.python_version = run_info.get("python", "3")
-
             if not self._entry_points:
+                _, ext = os.path.splitext(program_name)
+                if ext == ".py":
+                    entry_point = ["python", program_name]
+                elif ext == ".sh":
+                    command = os.environ.get("SHELL", "bash")
+                    entry_point = [command, program_name]
+                else:
+                    raise LaunchError(f"Unsupported entrypoint: {program_name}")
                 self.add_entry_point(entry_point)
             self.override_args = utils.merge_parameters(
                 self.override_args, run_info["args"]
@@ -301,95 +292,46 @@ class LaunchProject:
             assert utils._GIT_URI_REGEX.match(self.uri), (
                 "Non-wandb URI %s should be a Git URI" % self.uri
             )
-
             if not self._entry_points:
                 wandb.termlog(
-                    "Entry point for repo not specified, defaulting to main.py"
+                    "Entry point for repo not specified, defaulting to python main.py"
                 )
-                self.add_entry_point("main.py")
+                self.add_entry_point(["python", "main.py"])
             utils._fetch_git_repo(self.project_dir, self.uri, self.git_version)
 
 
 class EntryPoint:
     """An entry point into a wandb launch specification."""
 
-    def __init__(self, name: str, command: str):
+    def __init__(self, name: str, command: List[str]):
         self.name = name
         self.command = command
-        self.parameters: Dict[str, Any] = {}
 
-    def _validate_parameters(self, user_parameters: Dict[str, Any]) -> None:
-        missing_params = []
-        for name in self.parameters:
-            if name not in user_parameters and self.parameters[name].default is None:
-                missing_params.append(name)
-        if missing_params:
-            raise ExecutionError(
-                "No value given for missing parameters: %s"
-                % ", ".join(["'%s'" % name for name in missing_params])
-            )
-
-    def compute_parameters(
-        self, user_parameters: Optional[Dict[str, Any]]
-    ) -> Tuple[Dict[str, Optional[str]], Dict[str, Optional[str]]]:
-        """Validates and sanitizes parameters dict into expected dict format.
-
-        Given a dict mapping user-specified param names to values, computes parameters to
-        substitute into the command for this entry point. Returns a tuple (params, extra_params)
-        where `params` contains key-value pairs for parameters specified in the entry point
-        definition, and `extra_params` contains key-value pairs for additional parameters passed
-        by the user.
-        """
-        if user_parameters is None:
-            user_parameters = {}
-        # Validate params before attempting to resolve parameter values
-        self._validate_parameters(user_parameters)
-        final_params = {}
-        extra_params = {}
-
-        parameter_keys = list(self.parameters.keys())
-        for key in parameter_keys:
-            param_obj = self.parameters[key]
-            key_position = parameter_keys.index(key)
-            value = (
-                user_parameters[key]
-                if key in user_parameters
-                else self.parameters[key].default
-            )
-            final_params[key] = param_obj.compute_value(value, key_position)
-        for key in user_parameters:
-            if key not in final_params:
-                extra_params[key] = user_parameters[key]
-        return (
-            self._sanitize_param_dict(final_params),
-            self._sanitize_param_dict(extra_params),
-        )
-
-    def compute_command(self, user_parameters: Optional[Dict[str, Any]]) -> str:
+    def compute_command(self, user_parameters: Optional[Dict[str, Any]]) -> List[str]:
         """Converts user parameter dictionary to a string."""
-        params, extra_params = self.compute_parameters(user_parameters)
-        command_with_params = self.command.format(**params)
-        command_arr = [command_with_params]
-        command_arr.extend(
-            [
-                f"--{key} {value}" if value is not None else f"--{key}"
-                for key, value in extra_params.items()
-            ]
-        )
-        return " ".join(command_arr)
+        command_arr = []
+        command_arr += self.command
+        extras = compute_command_args(user_parameters)
+        command_arr += extras
+        return command_arr
 
-    @staticmethod
-    def _sanitize_param_dict(param_dict: Dict[str, Any]) -> Dict[str, Optional[str]]:
-        """Sanitizes a dictionary of parameters, quoting values, except for keys with None values."""
-        return {
-            (str(key)): (quote(str(value)) if value is not None else None)
-            for key, value in param_dict.items()
-        }
+
+def compute_command_args(parameters: Optional[Dict[str, Any]]) -> List[str]:
+    arr: List[str] = []
+    if parameters is None:
+        return arr
+    for key, value in parameters.items():
+        if value is not None:
+            arr.append(f"--{key}")
+            arr.append(quote(str(value)))
+        else:
+            arr.append(f"--{key}")
+    return arr
 
 
 def get_entry_point_command(
     entry_point: Optional["EntryPoint"], parameters: Dict[str, Any]
-) -> str:
+) -> List[str]:
     """Returns the shell command to execute in order to run the specified entry point.
 
     Arguments:
@@ -400,7 +342,7 @@ def get_entry_point_command(
         List of strings representing the shell command to be executed
     """
     if entry_point is None:
-        return ""
+        return []
     return entry_point.compute_command(parameters)
 
 
@@ -431,6 +373,7 @@ def create_project_from_spec(launch_spec: Dict[str, Any], api: Api) -> LaunchPro
         launch_spec.get("resource", "local"),
         launch_spec.get("resource_args", {}),
         launch_spec.get("cuda", None),
+        launch_spec.get("run_id", None),
     )
 
 
@@ -451,8 +394,10 @@ def fetch_and_validate_project(
         return launch_project
     if launch_project.source == LaunchSource.LOCAL:
         if not launch_project._entry_points:
-            wandb.termlog("Entry point for repo not specified, defaulting to main.py")
-            launch_project.add_entry_point("main.py")
+            wandb.termlog(
+                "Entry point for local directory not specified, defaulting to `python main.py`"
+            )
+            launch_project.add_entry_point(["python", "main.py"])
     else:
         launch_project._fetch_project_local(internal_api=api)
 
@@ -468,11 +413,6 @@ def fetch_and_validate_project(
     elif os.path.exists(os.path.join(launch_project.project_dir, "environment.yml")):
         launch_project.deps_type = "conda"
 
-    first_entry_point = list(launch_project._entry_points.keys())[0]
-    _logger.info("validating entrypoint parameters")
-    launch_project.get_entry_point(first_entry_point)._validate_parameters(
-        launch_project.override_args
-    )
     return launch_project
 
 
