@@ -1,32 +1,95 @@
 import logging
-from typing import Any, Dict, List, Optional
+import os
+from typing import Any, Dict, List, Optional, Tuple
 
 from wandb.apis.internal import Api
-from wandb.errors import ExecutionError
+from wandb.errors import ExecutionError, LaunchError
+import yaml
 
 from ._project_spec import create_project_from_spec, fetch_and_validate_project
 from .agent import LaunchAgent
+from .builder import loader as builder_loader
 from .runner import loader
 from .runner.abstract import AbstractRun
 from .utils import (
     construct_launch_spec,
+    LAUNCH_CONFIG_FILE,
     PROJECT_DOCKER_ARGS,
     PROJECT_SYNCHRONOUS,
+    resolve_build_and_registry_config,
 )
 
 _logger = logging.getLogger(__name__)
 
 
+def resolve_agent_config(
+    api: Api,
+    entity: Optional[str],
+    project: Optional[str],
+    max_jobs: Optional[int],
+    queues: Optional[List[str]],
+) -> Tuple[Dict[str, Any], Api]:
+    defaults = {
+        "entity": api.default_entity,
+        "max_jobs": 1,
+        "queues": ["default"],
+        "api_key": api.api_key,
+        "base_url": api.settings("base_url"),
+        "registry": {},
+        "build": {},
+    }
+
+    resolved_config: Dict[str, Any] = defaults
+    if os.path.exists(os.path.expanduser(LAUNCH_CONFIG_FILE)):
+        config = {}
+        with open(os.path.expanduser(LAUNCH_CONFIG_FILE)) as f:
+            try:
+                config = yaml.safe_load(f)
+                print(config)
+            except yaml.YAMLError as e:
+                raise LaunchError(f"Invalid launch agent config: {e}")
+        resolved_config.update(dict(config))
+    if os.environ.get("WANDB_PROJECT") is not None:
+        resolved_config.update({"project": os.environ.get("WANDB_PROJECT")})
+    if os.environ.get("WANDB_ENTITY") is not None:
+        resolved_config.update({"entity": os.environ.get("WANDB_ENTITY")})
+    if os.environ.get("WANDB_API_KEY") is not None:
+        resolved_config.update({"api_key": os.environ.get("WANDB_API_KEY")})
+    if os.environ.get("WANDB_LAUNCH_MAX_JOBS") is not None:
+        resolved_config.update({"max_jobs": os.environ.get("WANDB_LAUNCH_MAX_JOBS")})
+    if os.environ.get("WANDB_BASE_URL") is not None:
+        resolved_config.update({"base_url": os.environ.get("WANDB_BASE_URL")})
+
+    if project is not None:
+        resolved_config.update({"project": project})
+    if entity is not None:
+        resolved_config.update({"entity": entity})
+    if max_jobs is not None:
+        resolved_config.update({"max_jobs": int(max_jobs)})
+
+    if queues is not None:
+        resolved_config.update({"queues": queues})
+    if (
+        resolved_config["entity"] != defaults["entity"]
+        or resolved_config["api_key"] != defaults["api_key"]
+        or resolved_config["base_url"] != defaults["base_url"]
+    ):
+        settings = dict(
+            api_key=resolved_config["api_key"],
+            base_url=resolved_config["base_url"],
+            project=resolved_config["project"],
+            entity=resolved_config["entity"],
+        )
+        api = Api(default_settings=settings)
+
+    return resolved_config, api
+
+
 def create_and_run_agent(
     api: Api,
-    entity: str,
-    project: str,
-    queues: Optional[List[str]] = None,
-    max_jobs: float = None,
+    config: Dict[str, Any],
 ) -> None:
-    if queues is None:
-        queues = []
-    agent = LaunchAgent(entity, project, queues, max_jobs)
+    agent = LaunchAgent(api, config)
     agent.loop()
 
 
@@ -36,7 +99,7 @@ def _run(
     project: Optional[str],
     entity: Optional[str],
     docker_image: Optional[str],
-    entry_point: Optional[str],
+    entry_point: Optional[List[str]],
     version: Optional[str],
     parameters: Optional[Dict[str, Any]],
     resource: str,
@@ -45,6 +108,7 @@ def _run(
     synchronous: Optional[bool],
     cuda: Optional[bool],
     api: Api,
+    run_id: Optional[str],
 ) -> AbstractRun:
     """Helper that delegates to the project-running method corresponding to the passed-in backend."""
     launch_spec = construct_launch_spec(
@@ -61,6 +125,7 @@ def _run(
         resource_args,
         launch_config,
         cuda,
+        run_id,
     )
     launch_project = create_project_from_spec(launch_spec, api)
     launch_project = fetch_and_validate_project(launch_project, api)
@@ -68,15 +133,29 @@ def _run(
     # construct runner config.
     runner_config: Dict[str, Any] = {}
     runner_config[PROJECT_SYNCHRONOUS] = synchronous
+    build_config: Optional[Dict[str, Any]] = None
+    registry_config = None
     if launch_config is not None:
         given_docker_args = launch_config.get("docker", {}).get("args", {})
+        build_config = launch_config.get("build")
+        registry_config = launch_config.get("registry")
         runner_config[PROJECT_DOCKER_ARGS] = given_docker_args
     else:
         runner_config[PROJECT_DOCKER_ARGS] = {}
 
+    default_launch_config = None
+
+    if os.path.exists(os.path.expanduser(LAUNCH_CONFIG_FILE)):
+        with open(os.path.expanduser(LAUNCH_CONFIG_FILE)) as f:
+            default_launch_config = yaml.safe_load(f)
+
+    build_config, registry_config = resolve_build_and_registry_config(
+        default_launch_config, build_config, registry_config
+    )
+    builder = builder_loader.load_builder(build_config)
     backend = loader.load_backend(resource, api, runner_config)
     if backend:
-        submitted_run = backend.run(launch_project)
+        submitted_run = backend.run(launch_project, builder, registry_config)
         # this check will always pass, run is only optional in the agent case where
         # a run queue id is present on the backend config
         assert submitted_run
@@ -84,7 +163,7 @@ def _run(
     else:
         raise ExecutionError(
             "Unavailable backend {}, available backends: {}".format(
-                resource, ", ".join(loader.WANDB_RUNNERS.keys())
+                resource, ", ".join(loader.WANDB_RUNNERS)
             )
         )
 
@@ -92,7 +171,7 @@ def _run(
 def run(
     uri: Optional[str],
     api: Api,
-    entry_point: Optional[str] = None,
+    entry_point: Optional[List[str]] = None,
     version: Optional[str] = None,
     parameters: Optional[Dict[str, Any]] = None,
     name: Optional[str] = None,
@@ -104,6 +183,7 @@ def run(
     config: Optional[Dict[str, Any]] = None,
     synchronous: Optional[bool] = True,
     cuda: Optional[bool] = None,
+    run_id: Optional[str] = None,
 ) -> AbstractRun:
     """Run a W&B launch experiment. The project can be wandb uri or a Git URI.
 
@@ -131,6 +211,7 @@ def run(
         ``synchronous`` is True and the run fails, the current process will
         error out as well.
     cuda: Whether to build a CUDA-enabled docker image or not
+    run_id: ID for the run (To ultimately replace the :name: field)
 
 
     Example:
@@ -169,6 +250,7 @@ def run(
         synchronous=synchronous,
         cuda=cuda,
         api=api,
+        run_id=run_id,
     )
 
     return submitted_run_obj
