@@ -4,33 +4,34 @@ import logging
 import os
 import random
 import time
+from typing import Any, Callable, Generic, Optional, Tuple, Type, TypeVar
 
 from requests import HTTPError
 import wandb
 
+ExceptionPredicate = Callable[[Exception], bool]
+
 logger = logging.getLogger(__name__)
 
 
-def make_printer(msg):
-    def printer():
-        print(msg)
-
-    return printer
-
-
 class TransientError(Exception):
     """Exception type designated for errors that may only be temporary
 
     Can have its own message and/or wrap another exception.
     """
 
-    def __init__(self, msg=None, exc=None):
+    def __init__(
+        self, msg: Optional[str] = None, exc: Optional[BaseException] = None
+    ) -> None:
         super().__init__(msg)
         self.message = msg
         self.exception = exc
 
 
-class Retry:
+_R = TypeVar("_R")
+
+
+class Retry(Generic[_R]):
     """Creates a retryable version of a function.
 
     Calling this will call the passed function, retrying if any exceptions in
@@ -41,32 +42,35 @@ class Retry:
 
     def __init__(
         self,
-        call_fn,
-        retry_timedelta=None,
-        num_retries=None,
-        check_retry_fn=lambda e: True,
-        retryable_exceptions=None,
-        error_prefix="Network error",
-        retry_callback=None,
-    ):
+        call_fn: Callable[..., _R],
+        retry_timedelta: Optional[datetime.timedelta] = None,
+        num_retries: Optional[int] = None,
+        check_retry_fn: ExceptionPredicate = lambda e: True,
+        retryable_exceptions: Optional[Tuple[Type[Exception], ...]] = None,
+        error_prefix: str = "Network error",
+        retry_callback: Optional[Callable[[int, str], Any]] = None,
+        sleep_fn_for_testing: Callable[[float], None] = time.sleep,
+    ) -> None:
         self._call_fn = call_fn
         self._check_retry_fn = check_retry_fn
         self._error_prefix = error_prefix
         self._last_print = datetime.datetime.now() - datetime.timedelta(minutes=1)
         self._retry_timedelta = retry_timedelta
         self._num_retries = num_retries
-        self._retryable_exceptions = retryable_exceptions
-        if self._retryable_exceptions is None:
+        if retryable_exceptions is not None:
+            self._retryable_exceptions = retryable_exceptions
+        else:
             self._retryable_exceptions = (TransientError,)
         self._index = 0
         self.retry_callback = retry_callback
+        self._sleep_fn = sleep_fn_for_testing
 
     @property
-    def num_iters(self):
+    def num_iters(self) -> int:
         """The number of iterations the previous __call__ retried."""
         return self._num_iter
 
-    def __call__(self, *args, **kwargs):
+    def __call__(self, *args: Any, **kwargs: Any) -> _R:
         """Call the wrapped function, with retries.
 
         Arguments:
@@ -86,10 +90,12 @@ class Retry:
         if os.environ.get("WANDB_TEST"):
             num_retries = 0
 
-        sleep_base = kwargs.pop("retry_sleep_base", 1)
+        sleep_base: float = kwargs.pop("retry_sleep_base", 1)
 
-        # an extra function to allow performing more logic on the filtered exceptiosn
-        check_retry_fn = kwargs.pop("check_retry_fn", self._check_retry_fn)
+        # an extra function to allow performing more logic on the filtered exception
+        check_retry_fn: ExceptionPredicate = kwargs.pop(
+            "check_retry_fn", self._check_retry_fn
+        )
 
         sleep = sleep_base
         start_time = datetime.datetime.now()
@@ -141,7 +147,7 @@ class Retry:
                         )
                 # if wandb.env.is_debug():
                 #     traceback.print_exc()
-            time.sleep(sleep + random.random() * 0.25 * sleep)
+            self._sleep_fn(sleep + random.random() * 0.25 * sleep)
             sleep *= 2
             if sleep > self.MAX_SLEEP_SECONDS:
                 sleep = self.MAX_SLEEP_SECONDS
@@ -150,14 +156,17 @@ class Retry:
             self._num_iter += 1
 
 
-def retriable(*args, **kargs):
-    def decorator(fn):
-        retrier = Retry(fn, *args, **kargs)
+_F = TypeVar("_F", bound=Callable)
+
+
+def retriable(*args: Any, **kargs: Any) -> Callable[[_F], _F]:
+    def decorator(fn: _F) -> _F:
+        retrier: Retry[Any] = Retry(fn, *args, **kargs)
 
         @functools.wraps(fn)
-        def wrapped_fn(*args, **kargs):
+        def wrapped_fn(*args: Any, **kargs: Any) -> Any:
             return retrier(*args, **kargs)
 
-        return wrapped_fn
+        return wrapped_fn  # type: ignore
 
     return decorator
