@@ -8,6 +8,7 @@ For more on using `wandb.init()`, including code snippets, check out our
 [guide and FAQs](https://docs.wandb.ai/guides/track/launch).
 """
 import copy
+import json
 import logging
 import os
 import platform
@@ -72,6 +73,22 @@ def _maybe_mp_process(backend: Backend) -> bool:
     return False
 
 
+def _handle_launch_config(settings: "Settings") -> Dict[str, Any]:
+    launch_run_config = {}
+    if not settings.launch:
+        return launch_run_config
+    if os.environ.get("WANDB_CONFIG") is not None:
+        try:
+            launch_run_config = json.loads(os.environ.get("WANDB_CONFIG", "{}"))
+        except (ValueError, SyntaxError):
+            wandb.termwarn("Malformed WANDB_CONFIG, using original config")
+    elif settings.launch_config_path and os.path.exists(settings.launch_config_path):
+        with open(settings.launch_config_path) as fp:
+            launch_config = json.loads(fp.read())
+        launch_run_config = launch_config.get("overrides", {}).get("run_config")
+    return launch_run_config
+
+
 class _WandbInit:
     _init_telemetry_obj: telemetry.TelemetryRecord
 
@@ -79,6 +96,7 @@ class _WandbInit:
         self.kwargs = None
         self.settings = None
         self.sweep_config = None
+        self.launch_config = {}
         self.config = None
         self.run = None
         self.backend = None
@@ -190,20 +208,24 @@ class _WandbInit:
         )
 
         # merge config with sweep or sagemaker (or config file)
-        self.sweep_config = self._wl._sweep_config or dict()
+        self.sweep_config = dict()
+        sweep_config = self._wl._sweep_config or dict()
         self.config = dict()
         self.init_artifact_config = dict()
-        for config_data in sagemaker_config, self._wl._config, init_config:
+        for config_data in (
+            sagemaker_config,
+            self._wl._config,
+            init_config,
+        ):
             if not config_data:
                 continue
             # split out artifacts, since when inserted into
             # config they will trigger use_artifact
             # but the run is not yet upserted
-            for k, v in config_data.items():
-                if _is_artifact(v) or _is_artifact_string(v):
-                    self.init_artifact_config[k] = v
-                else:
-                    self.config.setdefault(k, v)
+            self._split_artifacts_from_config(config_data, self.config)
+
+        if sweep_config:
+            self._split_artifacts_from_config(sweep_config, self.sweep_config)
 
         monitor_gym = kwargs.pop("monitor_gym", None)
         if monitor_gym and len(wandb.patched["gym"]) == 0:
@@ -266,8 +288,12 @@ class _WandbInit:
 
             if settings._jupyter:
                 self._jupyter_setup(settings)
+        launch_config = _handle_launch_config(settings)
+        if launch_config:
+            self._split_artifacts_from_config(launch_config, self.launch_config)
 
         self.settings = settings
+
         # self.settings.freeze()
 
     def teardown(self):
@@ -277,6 +303,13 @@ class _WandbInit:
         for hook in self._teardown_hooks:
             hook.call()
 
+    def _split_artifacts_from_config(self, config_source, config_target):
+        for k, v in config_source.items():
+            if _is_artifact(v) or _is_artifact_string(v):
+                self.init_artifact_config[k] = v
+            else:
+                config_target.setdefault(k, v)
+
     def _enable_logging(self, log_fname, run_id=None):
         """Enables logging to the global debug log.
 
@@ -528,7 +561,12 @@ class _WandbInit:
 
         # resuming needs access to the server, check server_status()?
 
-        run = Run(config=self.config, settings=self.settings)
+        run = Run(
+            config=self.config,
+            settings=self.settings,
+            sweep_config=self.sweep_config,
+            launch_config=self.launch_config,
+        )
 
         # probe the active start method
         active_start_method: Optional[str] = None
@@ -675,10 +713,16 @@ class _WandbInit:
         self._wl._global_run_stack.append(run)
         self.run = run
 
+        run._handle_launch_artifact_overrides()
+        if (
+            self.settings.launch
+            and self.settings.launch_config_path
+            and os.path.exists(self.settings.launch_config_path)
+        ):
+            run._save(self.settings.launch_config_path)
         # put artifacts in run config here
         # since doing so earlier will cause an error
         # as the run is not upserted
-        run._populate_sweep_or_launch_config(self.sweep_config)
         for k, v in self.init_artifact_config.items():
             run.config.update({k: v}, allow_val_change=True)
 
