@@ -18,6 +18,7 @@ from typing import (
     Callable,
     Dict,
     FrozenSet,
+    ItemsView,
     Iterable,
     Mapping,
     no_type_check,
@@ -81,7 +82,7 @@ def _get_wandb_dir(root_dir: str) -> str:
     return os.path.expanduser(path)
 
 
-# fixme: should either return bool or error out. fix once confident.
+# todo: should either return bool or error out. fix once confident.
 def _str_as_bool(val: Union[str, bool]) -> bool:
     """
     Parse a string as a bool.
@@ -94,7 +95,7 @@ def _str_as_bool(val: Union[str, bool]) -> bool:
     except (AttributeError, ValueError):
         pass
 
-    # fixme: remove this and only raise error once we are confident.
+    # todo: remove this and only raise error once we are confident.
     wandb.termwarn(
         f"Could not parse value {val} as a bool. ",
         repeat=False,
@@ -156,7 +157,7 @@ def _get_program_relpath_from_gitrepo(
 @enum.unique
 class Source(enum.IntEnum):
     OVERRIDE: int = 0
-    BASE: int = 1  # fixme: audit this
+    BASE: int = 1  # todo: audit this
     ORG: int = 2
     ENTITY: int = 3
     PROJECT: int = 4
@@ -192,7 +193,7 @@ class Property:
           E.g. if `is_policy` is True, the smallest `Source` value takes precedence.
     """
 
-    # fixme: this is a temporary measure to bypass validation of the settings
+    # todo: this is a temporary measure to bypass validation of the settings
     #  whose validation was not previously enforced to make sure we don't brake anything.
     __strict_validate_settings = {
         "project",
@@ -234,7 +235,7 @@ class Property:
         self._is_policy = is_policy
         self._source = source
 
-        # fixme: this is a temporary measure to collect stats on failed preprocessing and validation
+        # todo: this is a temporary measure to collect stats on failed preprocessing and validation
         self.__failed_preprocessing: bool = False
         self.__failed_validation: bool = False
 
@@ -282,14 +283,14 @@ class Property:
         return value
 
     def _validate(self, value: Any) -> Any:
-        self.__failed_validation = False  # fixme: this is a temporary measure
+        self.__failed_validation = False  # todo: this is a temporary measure
         if value is not None and self._validator is not None:
             _validator = (
                 [self._validator] if callable(self._validator) else self._validator
             )
             for v in _validator:
                 if not v(value):
-                    # fixme: this is a temporary measure to bypass validation of certain settings.
+                    # todo: this is a temporary measure to bypass validation of certain settings.
                     #  remove this once we are confident
                     if self.name in self.__strict_validate_settings:
                         raise ValueError(
@@ -374,6 +375,8 @@ class Settings:
     _jupyter_path: str
     _jupyter_root: str
     _kaggle: bool
+    _live_policy_rate_limit: int
+    _live_policy_wait_time: int
     _noop: bool
     _offline: bool
     _os: str
@@ -385,6 +388,9 @@ class Settings:
     _service_transport: str
     _start_datetime: datetime
     _start_time: float
+    _stats_pid: int  # (internal) base pid for system stats
+    _stats_sample_rate_seconds: float
+    _stats_samples_to_average: int
     _tmp_code_dir: str
     _tracelog: str
     _unsaved_keys: Sequence[str]
@@ -409,6 +415,7 @@ class Settings:
     heartbeat_seconds: int
     host: str
     ignore_globs: Tuple[str]
+    init_timeout: int
     is_local: bool
     label_disable: bool
     launch: bool
@@ -469,6 +476,7 @@ class Settings:
     tmp_dir: str
     username: str
     wandb_dir: str
+    table_raise_on_max_row_limit_exceeded: bool
 
     def _default_props(self) -> Dict[str, Dict[str, Any]]:
         """
@@ -477,6 +485,9 @@ class Settings:
         Note that key names must be the same as the class attribute names.
         """
         return dict(
+            _disable_meta={"preprocessor": _str_as_bool},
+            _disable_stats={"preprocessor": _str_as_bool},
+            _disable_viewer={"preprocessor": _str_as_bool},
             _colab={
                 "hook": lambda _: "google.colab" in sys.modules,
                 "auto_hook": True,
@@ -499,7 +510,9 @@ class Settings:
                 "auto_hook": True,
             },
             _platform={"value": util.get_platform_name()},
-            _save_requirements={"value": True},
+            _save_requirements={"value": True, "preprocessor": _str_as_bool},
+            _stats_sample_rate_seconds={"value": 2.0},
+            _stats_samples_to_average={"value": 15},
             _tmp_code_dir={
                 "value": "code",
                 "hook": lambda x: self._path_convert(self.tmp_dir, x),
@@ -536,6 +549,7 @@ class Settings:
                 "value": tuple(),
                 "preprocessor": lambda x: tuple(x) if not isinstance(x, tuple) else x,
             },
+            init_timeout={"value": 30, "preprocessor": lambda x: int(x)},
             is_local={
                 "hook": (
                     lambda _: self.base_url != "https://api.wandb.ai"
@@ -634,11 +648,17 @@ class Settings:
             },
             system_sample={"value": 15},
             system_sample_seconds={"value": 2},
+            table_raise_on_max_row_limit_exceeded={
+                "value": False,
+                "preprocessor": _str_as_bool,
+            },
             timespec={
                 "hook": (
-                    lambda _: datetime.strftime(self._start_datetime, "%Y%m%d_%H%M%S")
-                    if self._start_time and self._start_datetime
-                    else None
+                    lambda _: (
+                        datetime.strftime(self._start_datetime, "%Y%m%d_%H%M%S")
+                        if self._start_datetime
+                        else None
+                    )
                 ),
                 "auto_hook": True,
             },
@@ -817,7 +837,7 @@ class Settings:
             r"\.?"  # may have a trailing dot
         )
         # host_re = "(" + hostname_re + domain_re + tld_re + "|localhost)"
-        # fixme?: allow hostname to be just a hostname (no tld)?
+        # todo?: allow hostname to be just a hostname (no tld)?
         host_re = "(" + hostname_re + domain_re + f"({tld_re})?" + "|localhost)"
 
         regex = re.compile(
@@ -910,6 +930,9 @@ class Settings:
         return f"{project_url}{query}"
 
     def _run_url(self) -> str:
+        """
+        Return the run url.
+        """
         project_url = self._project_url_base()
         if not all([project_url, self.run_id]):
             return ""
@@ -917,13 +940,25 @@ class Settings:
         query = self._get_url_query_string()
         return f"{project_url}/runs/{quote(self.run_id)}{query}"
 
-    def _start_run(self) -> None:
+    def _set_run_start_time(self, source: int = Source.BASE) -> None:
+        """
+        Set the time stamps for the settings.
+        Called once the run is initialized.
+        """
         time_stamp: float = time.time()
         datetime_now: datetime = datetime.fromtimestamp(time_stamp)
         object.__setattr__(self, "_Settings_start_datetime", datetime_now)
         object.__setattr__(self, "_Settings_start_time", time_stamp)
+        self.update(
+            _start_datetime=datetime_now,
+            _start_time=time_stamp,
+            source=source,
+        )
 
     def _sweep_url(self) -> str:
+        """
+        Return the sweep url.
+        """
         project_url = self._project_url_base()
         if not all([project_url, self.sweep_id]):
             return ""
@@ -935,7 +970,7 @@ class Settings:
         self.__frozen: bool = False
         self.__initialized: bool = False
 
-        # fixme: this is collect telemetry on validation errors and unexpected args
+        # todo: this is collect telemetry on validation errors and unexpected args
         # values are stored as strings to avoid potential json serialization errors down the line
         self.__preprocessing_warnings: Dict[str, str] = dict()
         self.__validation_warnings: Dict[str, str] = dict()
@@ -985,7 +1020,7 @@ class Settings:
                     ),
                 )
 
-            # fixme: this is to collect stats on preprocessing and validation errors
+            # todo: this is to collect stats on preprocessing and validation errors
             if self.__dict__[prop].__dict__["_Property__failed_preprocessing"]:
                 self.__preprocessing_warnings[prop] = str(self.__dict__[prop]._value)
             if self.__dict__[prop].__dict__["_Property__failed_validation"]:
@@ -996,7 +1031,7 @@ class Settings:
         # allow only explicitly defined arguments
         if unexpected_arguments:
 
-            # fixme: remove this and raise error instead once we are confident
+            # todo: remove this and raise error instead once we are confident
             self.__unexpected_args.update(unexpected_arguments)
             wandb.termwarn(
                 f"Ignoring unexpected arguments: {unexpected_arguments}. "
@@ -1023,7 +1058,7 @@ class Settings:
         # done with init, use self.update() to update attributes from now on
         self.__initialized = True
 
-        # fixme? freeze settings to prevent accidental changes
+        # todo? freeze settings to prevent accidental changes
         # self.freeze()
 
     def __str__(self) -> str:
@@ -1049,7 +1084,7 @@ class Settings:
         """
         Ensure that a copy of the settings object is a truly deep copy
 
-        Note that the copied object will not be frozen  fixme? why is this needed?
+        Note that the copied object will not be frozen  todo? why is this needed?
         """
         # get attributes that are instances of the Property class:
         attributes = {k: v for k, v in self.__dict__.items() if isinstance(v, Property)}
@@ -1099,13 +1134,37 @@ class Settings:
 
     def update(
         self,
-        settings: Optional[Dict[str, Any]] = None,
+        settings: Optional[Union[Dict[str, Any], "Settings"]] = None,
         source: int = Source.OVERRIDE,
         **kwargs: Any,
     ) -> None:
         """Update individual settings using the Property.update() method."""
         if "_Settings__frozen" in self.__dict__ and self.__frozen:
             raise TypeError("Settings object is frozen")
+
+        if isinstance(settings, Settings):
+            # If a Settings object is passed, detect the settings that differ
+            # from defaults, collect them into a dict, and apply them using `source`.
+            # This comes up in `wandb.init(settings=wandb.Settings(...))` and
+            # seems like the behavior that the user would expect when calling init that way.
+            defaults = Settings()
+            settings_dict = dict()
+            for k, v in settings.__dict__.items():
+                if isinstance(v, Property):
+                    if v._value != defaults.__dict__[k]._value:
+                        settings_dict[k] = v._value
+            # todo: store warnings from the passed Settings object, if any,
+            #  to collect telemetry on validation errors and unexpected args.
+            #  remove this once strict checking is enforced.
+            for attr in (
+                "_Settings__unexpected_args",
+                "_Settings__preprocessing_warnings",
+                "_Settings__validation_warnings",
+            ):
+                getattr(self, attr).update(getattr(settings, attr))
+            # replace with the generated dict
+            settings = settings_dict
+
         # add kwargs to settings
         settings = settings or dict()
         # explicit kwargs take precedence over settings
@@ -1121,7 +1180,7 @@ class Settings:
         for key, value in settings.items():
             self.__dict__[key].update(value, source)
 
-            # fixme: this is to collect stats on preprocessing and validation errors
+            # todo: this is to collect stats on preprocessing and validation errors
             if self.__dict__[key].__dict__["_Property__failed_preprocessing"]:
                 self.__preprocessing_warnings[key] = str(self.__dict__[key]._value)
             else:
@@ -1132,6 +1191,12 @@ class Settings:
             else:
                 self.__validation_warnings.pop(key, None)
 
+    def items(self) -> ItemsView[str, Any]:
+        return self.make_static().items()
+
+    def get(self, key: str, default: Any = None) -> Any:
+        return self.make_static().get(key, default)
+
     def freeze(self) -> None:
         object.__setattr__(self, "_Settings__frozen", True)
 
@@ -1166,7 +1231,7 @@ class Settings:
             # note that only the same/higher priority settings are propagated
             self.update({k: v._value}, source=v.source)
 
-        # fixme: this is to pass on info on unexpected args in settings
+        # todo: this is to pass on info on unexpected args in settings
         if settings.__dict__["_Settings__unexpected_args"]:
             self.__dict__["_Settings__unexpected_args"].update(
                 settings.__dict__["_Settings__unexpected_args"]
@@ -1185,6 +1250,11 @@ class Settings:
                 config[k] = config[k].split(",")
         return config
 
+    def _apply_base(self, pid: int, _logger: Optional[_EarlyLogger] = None) -> None:
+        if _logger is not None:
+            _logger.info(f"Configure stats pid to {pid}")
+        self.update({"_stats_pid": pid}, source=Source.SETUP)
+
     def _apply_config_files(self, _logger: Optional[_EarlyLogger] = None) -> None:
         # TODO(jhr): permit setting of config in system and workspace
         if self.settings_system is not None:
@@ -1240,7 +1310,6 @@ class Settings:
             _logger.info(
                 f"Loading settings from environment variables: {_redact_dict(env)}"
             )
-
         self.update(env, source=Source.ENV)
 
     def _infer_settings_from_environment(
@@ -1308,7 +1377,8 @@ class Settings:
         if os.path.exists("/usr/local/cuda/version.txt"):
             with open("/usr/local/cuda/version.txt") as f:
                 settings["_cuda"] = f.read().split(" ")[-1].strip()
-        settings["_args"] = sys.argv[1:]
+        if not self._jupyter:
+            settings["_args"] = sys.argv[1:]
         settings["_os"] = platform.platform(aliased=True)
         settings["_python"] = platform.python_version()
         # hack to make sure we don't hang on windows
