@@ -1,13 +1,12 @@
-#
 """watch."""
 
 import logging
 import os
+from typing import Optional
 
 import wandb
 
 from .lib import telemetry
-from .lib.ipython import _get_python_type
 
 logger = logging.getLogger("wandb")
 
@@ -15,7 +14,12 @@ _global_watch_idx = 0
 
 
 def watch(
-    models, criterion=None, log="gradients", log_freq=1000, idx=None, log_graph=False
+    models,
+    criterion=None,
+    log: Optional[str] = "gradients",
+    log_freq: int = 1000,
+    idx: Optional[int] = None,
+    log_graph: bool = False,
 ):
     """Hooks into the torch model to collect gradients and the topology.
 
@@ -30,7 +34,7 @@ def watch(
         log_graph: (boolean) log graph topology
 
     Returns:
-        `wandb.Graph` The graph object that will populate after the first backward pass
+        `wandb.Graph`: The graph object that will populate after the first backward pass
 
     Raises:
         ValueError: If called before `wandb.init` or if any of models is not a torch.nn.Module.
@@ -48,8 +52,6 @@ def watch(
     if wandb.run is None:
         raise ValueError("You must call `wandb.init` before calling watch")
 
-    in_jupyter = _get_python_type() != "python"
-
     log_parameters = False
     log_gradients = True
     if log == "all":
@@ -86,19 +88,16 @@ def watch(
             # TODO: this makes ugly chart names like gradients/graph_1conv1d.bias
             prefix = "graph_%i" % global_idx
 
-        wandb.run.history.torch.add_log_hooks_to_pytorch_module(
+        wandb.run._torch.add_log_hooks_to_pytorch_module(
             model,
             log_parameters=log_parameters,
             log_gradients=log_gradients,
             prefix=prefix,
             log_freq=log_freq,
-            jupyter_run=wandb.run if in_jupyter else None,
         )
 
         if log_graph:
-            graph = wandb.run.history.torch.hook_torch(
-                model, criterion, graph_idx=global_idx
-            )
+            graph = wandb.run._torch.hook_torch(model, criterion, graph_idx=global_idx)
             graphs.append(graph)
             # NOTE: the graph is set in run.summary by hook_torch on the backward pass
     return graphs
@@ -118,9 +117,9 @@ def unwatch(models=None):
                 wandb.termwarn("%s model has not been watched" % model)
             else:
                 for name in model._wandb_hook_names:
-                    wandb.run.history.torch.unhook(name)
+                    wandb.run._torch.unhook(name)
                 delattr(model, "_wandb_hook_names")
                 # TODO: we should also remove recursively model._wandb_watch_called
 
     else:
-        wandb.run.history.torch.unhook_all()
+        wandb.run._torch.unhook_all()
